@@ -60,7 +60,7 @@ typedef struct {
 
 typedef struct {
   Atom net_wm_state_atom;
-  Atom new_wm_state_modal_atom;
+  Atom net_wm_state_modal_atom;
   Atom net_wm_state_sticky_atom;
   Atom net_wm_state_maximized_vert_atom;
   Atom net_wm_state_maximized_horz_atom;
