@@ -109,7 +109,7 @@
   return [description copy];
 }
 
-- copyWithZone: (NSZone *)zone
+- (id)copyWithZone: (NSZone *)zone
 {
   GSGState *new = (GSGState *)NSCopyObject(self, 0, zone);  
   /* Do a deep copy since gstates are isolated from each other */
@@ -143,6 +143,36 @@
   ASSIGN(pattern, image);
 }
 
+- (void) setShouldAntialias: (BOOL)antialias
+{
+  _antialias = antialias;
+}
+
+- (BOOL) shouldAntialias
+{
+  return _antialias;
+}
+
+- (NSPoint) patternPhase
+{
+  return _patternPhase;
+}
+
+- (void) setPatternPhase: (NSPoint)phase
+{
+  _patternPhase = phase;
+}
+
+- (NSCompositingOperation) compositingOperation
+{
+  return _compositingOperation;
+}
+
+- (void) setCompositingOperation: (NSCompositingOperation)operation
+{
+  _compositingOperation = operation;
+}
+
 // This is only a fall back, the method should not be called any more.
 - (void) compositeGState: (GSGState *)source
                 fromRect: (NSRect)aRect
@@ -949,7 +979,7 @@ typedef enum {
     ASSIGN(path, [path bezierPathByFlatteningPath]);
 }
 
-- (void) DPSinitclip;
+- (void) DPSinitclip
 {
   [self subclassResponsibility: _cmd];
 }
@@ -1276,31 +1306,38 @@ typedef enum {
 - (void) _fillRect: (NSRect)rect withPattern: (NSImage*)color_pattern
 {
   NSSize size;
-  CGFloat x;
-  CGFloat y;
   NSAffineTransform *ictm;
+  NSPoint patternPhase, startPoint, endPoint, point;
 
   // The coordinates we get here are already in device space,
   // but compositeToPoint needs user space coordinates
   ictm = [ctm copyWithZone: [self zone]];
   [ictm invert];
 
-  size = [pattern size];
-  y = floor(NSMinY(rect) / size.height) * size.height;
+  size = [color_pattern size];
+  patternPhase = [self patternPhase];
 
-    while (y < NSMaxY(rect))
+  if (!NSEqualPoints(patternPhase, NSZeroPoint))
     {
-      x = floor(NSMinX(rect) / size.width) * size.width;
-      while (x < NSMaxX(rect))
+      // patternPhase % pattern size
+      patternPhase.x -= floor(patternPhase.x / size.width) * size.width;
+      patternPhase.y -= floor(patternPhase.y / size.height) * size.height;
+    }
+
+  startPoint = NSMakePoint(floor((NSMinX(rect) - patternPhase.x) / size.width) * size.width 
+                           + patternPhase.x,
+                           floor((NSMinY(rect) - patternPhase.y) / size.height) * size.height
+                           + patternPhase.y);
+
+  endPoint = NSMakePoint(NSMaxX(rect), NSMaxY(rect));
+
+  for (point.y = startPoint.y; point.y < endPoint.y; point.y += size.height)
+    {
+      for (point.x = startPoint.x; point.x < endPoint.x; point.x += size.width)
         {
-          NSPoint p = NSMakePoint(x, y);
-          
-          p = [ictm transformPoint: p];
-	  [color_pattern compositeToPoint: p
-                         operation: NSCompositeSourceOver];
-	  x += size.width;
+	  [color_pattern compositeToPoint: [ictm transformPoint: point]
+                                operation: NSCompositeSourceOver];
         }
-      y += size.height;
     }
   RELEASE(ictm);
 }
